#!/bin/bash
#------------------------------------------------
# Author(s): Geaaru, geaaru@gmail.com
# $Id: 552efebd13e28713e7ef6ea144a5923e5238ac93 $
# License: GPL 2.0
#------------------------------------------------

# psql_set_auth_var
psql_set_auth_var () {

  local db=$1
  local user=$2
  local pwd=$3
  local host=$4
  local schema=$5
  local v_host=""
  local v_schema=""

  if [ ! -z "$host" ] ; then
    v_host="-h $host"
  fi

  if [ ! -z "$schema" ] ; then
    v_schema="-v schema=${schema}"
  fi

  export PGPASSWORD="$pwd"
  # TODO: handle ~/.pgpass file

  psql_auth="$v_host $v_schema -U $user -w $db"

  [[ $DEBUG && $DEBUG == true ]] && echo "Use '$psql_auth'"

  return 0
}
# psql_set_auth_var_end


# psql_psql_cmd_4var
psql_cmd_4var () {

  set -f
  local var=$1
  local cmd="$2"
  local rm_lf=$3
  local avoid_tmz=$4
  local v=""
  local opts=""
  # Set separator
  opts="${opts} -P fieldsep='|'"
  # Disable footer
  opts="${opts} -P footer=off"
  # Tuple only
  opts="${opts} -t" # equals to -P tuples_only=true
  # Quiet mode
  # opts="${opts} -q"


  local tz="
\o /dev/null
SET timezone = '$POSTGRESQL_TMZ';
\o
"
  if [[ -n "$avoid_tmz" && x"$avoid_tmz" == x"1" ]] ; then
    tz=""
  fi
  if [[ ! -z "$POSTGRESQL_IGNORE_TMZ" && $POSTGRESQL_IGNORE_TMZ -eq 1 ]] ; then
    tz=""
  fi

  if [ -z "$POSTGRESQL_SHOW_COLUMNS" ] ; then
    # Disable columns as default
    opts="${opts} -P columns=off"
  fi

  if [ -z "${POSTGRESQL_FORMAT}" ] ; then
    # Format unaligned
    opts="${opts} -P format=unaligned"
  fi

  [[ $DEBUG && $DEBUG == true ]] && echo -en "Connection options: $POSTGRESQL_CLIENT $opts $POSTGRESQL_EXTRA_OPTIONS $psql_auth\n"

  v=$($POSTGRESQL_CLIENT $opts $POSTGRESQL_EXTRA_OPTIONS $psql_auth 2>&1 <<EOF
$tz
$cmd;
\q
EOF
)

  local ans=$?

  [[ $DEBUG && $DEBUG == true ]] && echo -en "$cmd ==> $v ($ans)\n"

  if [[ $ans -eq 0 && -n "$v" ]] ; then

    if [[ -n $rm_lf ]] ; then

      v=`echo $v | sed 's/\n//g'`

    fi

  fi

#  declare -x "$var"="$v"
  #read -r "$var" <<< "$v"
  eval "$var=\$v"

  set +f

  return $ans
}
# psql_psql_cmd_4var_end

# vim: syn=sh filetype=sh
